@@ -1,6 +1,7 @@
 /*
 * Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
 * Copyright 2014  Hugo Pereira Da Costa <hugo.pereira@free.fr>
+* Copyright 2018  Vlad Zagorodniy <vladzzag@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
@@ -29,7 +30,7 @@
 #include "breezebutton.h"
 #include "breezesizegrip.h"
 
-#include "breezeboxshadowhelper.h"
+#include "breezeboxshadowrenderer.h"
 
 #include <KDecoration2/DecoratedClient>
 #include <KDecoration2/DecorationButtonGroup>
@@ -729,43 +730,37 @@ namespace Breeze
                 return c;
             };
 
-            // In order to properly render a box shadow with a given radius `shadowSize`,
-            // the box size should be at least `2 * QSize(shadowSize, shadowSize)`.
-            const int shadowSize = qMax(params.shadow1.radius, params.shadow2.radius);
-            const QRect box(shadowSize, shadowSize, 2 * shadowSize + 1, 2 * shadowSize + 1);
-            const QRect rect = box.adjusted(-shadowSize, -shadowSize, shadowSize, shadowSize);
+            const QSize boxSize = BoxShadowRenderer::calculateMinimumBoxSize(params.shadow1.radius)
+                .expandedTo(BoxShadowRenderer::calculateMinimumBoxSize(params.shadow2.radius));
 
-            QImage shadow(rect.size(), QImage::Format_ARGB32_Premultiplied);
-            shadow.fill(Qt::transparent);
-
-            QPainter painter(&shadow);
-            painter.setRenderHint(QPainter::Antialiasing);
+            BoxShadowRenderer shadowRenderer;
+            shadowRenderer.setBorderRadius(Metrics::Frame_FrameRadius + 0.5);
+            shadowRenderer.setBoxSize(boxSize);
+            shadowRenderer.setDevicePixelRatio(1.0); // TODO: Create HiDPI shadows?
 
             const qreal strength = static_cast<qreal>(g_shadowStrength) / 255.0;
-
-            // Draw the "shape" shadow.
-            BoxShadowHelper::boxShadow(
-                &painter,
-                box,
-                params.shadow1.offset,
-                params.shadow1.radius,
+            shadowRenderer.addShadow(params.shadow1.offset, params.shadow1.radius,
                 withOpacity(g_shadowColor, params.shadow1.opacity * strength));
-
-            // Draw the "contrast" shadow.
-            BoxShadowHelper::boxShadow(
-                &painter,
-                box,
-                params.shadow2.offset,
-                params.shadow2.radius,
+            shadowRenderer.addShadow(params.shadow2.offset, params.shadow2.radius,
                 withOpacity(g_shadowColor, params.shadow2.opacity * strength));
 
+            QImage shadowTexture = shadowRenderer.render();
+
+            QPainter painter(&shadowTexture);
+            painter.setRenderHint(QPainter::Antialiasing);
+
+            const QRect outerRect = shadowTexture.rect();
+
+            QRect boxRect(QPoint(0, 0), boxSize);
+            boxRect.moveCenter(outerRect.center());
+
             // Mask out inner rect.
             const QMargins padding = QMargins(
-                shadowSize - Metrics::Shadow_Overlap - params.offset.x(),
-                shadowSize - Metrics::Shadow_Overlap - params.offset.y(),
-                shadowSize - Metrics::Shadow_Overlap + params.offset.x(),
-                shadowSize - Metrics::Shadow_Overlap + params.offset.y());
-            const QRect innerRect = rect - padding;
+                boxRect.left() - outerRect.left() - Metrics::Shadow_Overlap - params.offset.x(),
+                boxRect.top() - outerRect.top() - Metrics::Shadow_Overlap - params.offset.y(),
+                outerRect.right() - boxRect.right() - Metrics::Shadow_Overlap + params.offset.x(),
+                outerRect.bottom() - boxRect.bottom() - Metrics::Shadow_Overlap + params.offset.y());
+            const QRect innerRect = outerRect - padding;
 
             painter.setPen(Qt::NoPen);
             painter.setBrush(Qt::black);
@@ -788,8 +783,8 @@ namespace Breeze
 
             g_sShadow = QSharedPointer<KDecoration2::DecorationShadow>::create();
             g_sShadow->setPadding(padding);
-            g_sShadow->setInnerShadowRect(QRect(shadow.rect().center(), QSize(1, 1)));
-            g_sShadow->setShadow(shadow);
+            g_sShadow->setInnerShadowRect(QRect(outerRect.center(), QSize(1, 1)));
+            g_sShadow->setShadow(shadowTexture);
         }
 
         setShadow(g_sShadow);
